/* A simple 4 bit ALU.
The input busses are A and B.
C is a control bus which defines the operation to be performed.
S is the output bus.
CYIN is a carry/borrow input.  CYOUT is a carry/borrow  output.
Both are defined only for arithmetic add and subtract.
In order to execute the arithmetic add and subtract with 5 bits,
the input and output ports A,B and S are expanded to 5 bits groups 
by the names AX,BX and SX.
CYIN is expanded to a 5 bit group by the name CYINX.

Results are minimized and printed in equations format.
The design is also simulated by testing a few cases.
*/

/******************* declarations ********************/
input A[4], B[4], C[4];
input  CYIN;
output  S[4],CYOUT;
boolean AX[5],BX[5],CYINX[5];
group SX = [CYOUT,S];

/*********** definition of circuit behavior ************/
AX = A;  BX = B;  CYINX = gr(5,CYIN);   /* expand to 5 bit groups */
.switch (C)
{   case 0: S = A & B;                  /* and */
    case 1: S = A & !B;                 /* and not */
    case 2: S = A ^ B;                  /* XOR */
    case 3: S = A | B;                  /* or */
    case 4: S = !A;                     /* not A */
    case 5: S = !B;                     /* not B */
    case 6: S = A >> 1;                 /* shift A right */
    case 7: S = A << 1;                 /* shift A left */
    case 8: SX = AX + BX + CYINX;       /* arithmetic add*/
    case 9: SX = AX - BX - CYINX;       /* arithmetic subtract*/
}
/***************** minimizing and simulating ***********/
minimize(SX);
print(SX);

/**************** simulating ***************************/
#define prntdec printf(2,"A=%u  B=%u CYIN=%b  C=%u  S=%u\n",A,B,CYIN,C,S)
#define prntbin printf(2,"A=%b  B=%b  C=%u  S=%b\n",A,B,C,S)
printf(2,"ADD:\n");
values(A = @5, B = @3, C = @8, CYIN = 1); prntdec;
values(CYIN=0); prntdec;
printf(2,"SUBTRACT:\n");
values(C = @9); prntdec;
values(A = @9, B = @7); prntdec;
printf(2,"SHIFT A LEFT:\n");
values(C = @7); prntbin;
printf(2,"SHIFT A RIGHT:\n");
values(C = @6); prntbin;
printf(2,"A xor B:\n");
values(C = @2); prntbin;

end;

