/* General comment:  Starting from version 1.3 and above, the language 
supports the operators + and - as legal operators on groups.
However, The functions add, subtract and negate were left here for
backward compatibility and also as examples.
*/

/********************************************************************
                      add(a,b,cyin,cyout)

This function performs the operation of an adder with a,b as input
buses, cyin as a carry input, and cyout as a carry output. 
(The adder is doing the operation c = a + b + cyin).  The
buses a and b may have different sizes (number of elements). The
return value of the function is a group representing the output  bus
of the adder.  The number of elements in the result is the same as
the input bus with the larger number of elements.  If you don't use
either cyin or cyout, actual parameters of 0 can be used.  For
example, you can write:

C = add(A,B,0,0);

This will assign to the bus C the equations of an adder without carry
in or carry out. (It calculates only: C = A + B)
An example showing how to use this function can be found in the
tutorial section of the manual.
*********************************************************************/

group add(a,b,cyin,cyout)
bool a[],b[];    /* inputs */
bool   cyin,  /* carry in */
       cyout; /* carry out */
{
   bool carry;  /* internal carry of each one-bit adder*/
   bool aa[],bb[]; /* used as inputs. size(aa) >= size(bb) */
   bool c[]; /* used for result */
   int ii;

   if(size(a) >= size(b) ) { aa = a; bb = b;}
   else { aa = b; bb = a; }
   carry = cyin;
   for( ii = 0,size(bb)-1 )   /* evaluate one-bit adder n times */
   {  c[ii] = aa[ii] ^ bb[ii] ^ carry ; /* output bit no. ii */
      carry <= aa[ii] & bb[ii] | (aa[ii] | bb[ii]) & carry ;
   }
   for( ii = size(bb),size(aa)-1)
   {  c[ii] = aa[ii] ^ carry ; /* output bit no. ii */
      carry <= aa[ii] & carry ;
   }
   cyout := carry;
   return c;
}



/********************************************************************
                      subtract(x,y,brin,brout)

This function performs the operation of a subtracter with input 
buses x and y, brin as a borrow input, and brout as a borrow output. 
(The subtracter is doing the operation z = x - y - brin).  The buses
a and b may have different sizes (number of elements). The return
value of the function is a group representing the output bus of the
subtracter.  The number of elements in the result is the same as the
bus with the larger number of elements. If you don't use either brin
or brout, actual parameters of 0 can be used.  For example, you can
write:

C = subtract(A,B,0,0);

This will assign to the bus C the equations of a subtracter without
borrow in or borrow out. (It calculates only: C = A - B).    Note
that the subtracter is actually implemented with an adder by
inverting some of the signals. An example showing how to use this
function can be found in the tutorial section of the manual.
*********************************************************************/

group subtract(x,y,brin,brout)
bool x[],y[],    /* inputs */
     brin,       /* borrow input */
     brout;      /* borrow output */
{
    bool z[],cyout;
    /* if y has less elements than x, fill upper elements with 0  */
    if(size(x) > size(y))
        z = add(x,!gr(size(x),y),!brin,cyout);
    else
        z = add(x,!y,!brin,cyout);
    brout := !cyout;
    return z;
}




/********************************************************************
                      mult(a,b)

This function performs the operation of a multiplier with input 
buses a and b, both representing unsigned integer numbers.  (The
multiplier is doing the operation c = a * b).  The buses a and b may
have different sizes (number of elements). The return value of the
function is a group representing the output bus of the multiplier. 
The number of elements in the result is the sum of the number of
elements in the buses a and b.
Example:

C = mult(A,B);

This will assign to the bus C the boolean equations of a multiplier
that calculates C = A * B, assuming that both A and B represent
unsigned numbers.
*********************************************************************/

group mult(a,b)
bool a[],b[];
{
   bool sum[]; /* intermediate results */
   int kk,
       szb, /* size of b */
       rsz; /* size of result */
   
   szb = size(b);
   rsz = szb + size(a);
   sum = rsz@0;
   for ( kk = szb-1 , 0 , -1 )
      sum <=  gr(rsz,sum,0) + (a & b[kk]) ;
   return sum;
}



/********************************************************************
                      smult(a,b)

This function performs the operation of a multiplier with input 
buses a and b, both representing signed (2's complement) integer
numbers.  (The multiplier is doing the operation c = a * b).  The
buses a and b may have different sizes (number of elements). The
return value of the function is a group representing the output bus
of the multiplier.  The number of elements in the result is the sum of
the number of elements in the buses a and b.
The group returned as a result also represents a signed bus.
Example:

C = smult(A,B);

This will assign to the bus C the boolean equations of a multiplier
that calculates C = A * B, assuming that A,B,C all represent
signed numbers.
*********************************************************************/

group smult(a,b)
bool a[],b[];
{
   bool res[]; /* holds the result */
   int sza1, /* size of a minus 1 */
       szb1; /* size of b minus 1 */
   
   szb1 = size(b) - 1;
   sza1 = size(a) - 1;
   res = !a[sza1] & ! b[szb1] & mult(a,b) |
          a[sza1] & ! b[szb1] & -mult(-a,b) |
         !a[sza1] &   b[szb1] & -mult(a,-b) |
          a[sza1] &   b[szb1] & mult(-a,-b) ;
         
   return res ;
}



/********************************************************************
                      neg(x)

This function performs a 2's complement negation.
The result returned from neg is a bus with the same number of
elements as in x.
For example:

Z = neg(X);

X and Z are both buses representing signed integer numbers.
(It does: Z = -X )
*********************************************************************/


group neg(x)
bool x[];
{  return add(~x,1@1,0,0); }


