/********************************************************************
                      ffd(q,j,k)
                      gffd(q,j,k)

ffd is a function that converts a design done for a JK type flip flop
to a design with D type flip flop.  The inputs to the function are:
j,k  - boolean variables that have the equations (that were already
calculated) of the J and K inputs of the flip-flop.
q - is an input variable that shows the current state of the
flip-flop (Its output)
The function returns the equation of the D input to the flip-flop.

gffd is used for designs with multiple flip-flops.
If you have more than one flip-flop to convert, (A group of
flip-flops) you can use gffd.   gffd requires that q,j,k are all
groups and it returns a group as a result.

NOTE:  Most designs start with a specification for a D type flip-flop
(Which is more easy to design with) and then, if required, converted
to a JK type flip-flop (for better hardware efficiency). This
function can therefore be used in the simulation phase to calculate
the next state of the JK flip flop based on his j,k inputs and its
current state q.
********************************************************************/

bool ffd(q,j,k)
bool q,j,k;
{ return !k & q | j & !q; }

group gffd(q,j,k)
bool q[],j[],k[];
{ return !k & q | j & !q; }

   


/********************************************************************
                      isidentical(a,b)
                      gisidentical(ga,gb)

isidentical checks that the two boolean variables a and b contain
identical values. Two boolean values are considered as identical if
each minterm maps to the same point, whether it is one, zero, or
don't care.  It does not guarantee however, that the two values are
represented by the same product terms.
If a and b are identical, then the value 1 is returned, otherwise,
the value 0 is returned.

gisidentical is  doing the same for two group variables.  the two
groups ga and gb are considered as identical if they have the same
number of elements, and if every two corresponding elements are
identical.
********************************************************************/

int isidentical(a,b)
bool a,b;
{
   if(!iszero(one(a) & !one(b) ) ) return 0;
   if(!iszero(!one(a) & one(b) ) ) return 0;
   if(!iszero(zero(a) & !zero(b) ) ) return 0;
   if(!iszero(!zero(a) & zero(b) ) ) return 0;
   return 1; /* identical */
}


int gisidentical(ga,gb)
bool ga[],gb[];
{
   int ii;
   
   if(size(ga) != size(gb)) return 0; /* different size */
   for(ii = 0,size(ga)-1) if(!isidentical(ga[ii],gb[ii])) return 0;
   return 1;
}



/********************************************************************
                      dontcare(y)

Returns the don't care set of the boolean expression y .
For example, if y is fully specified, then the returned value will be
a global 0.  As another example, if y is a boolean variable that was
never assigned any value, the returned value will be a global 1.
********************************************************************/

bool dontcare(y)
bool y;
{ return !(one(y) | zero(y)); }


/********************************************************************
                      and_all(x)
                      or_all(x)
                      xor_all(x)

Each of these functions accept as an input parameter a group, and
returns a scalar boolean value.
The function and_all(x) performs the AND operation between all
elements of x.
Similarly, or_all(x) and xor_all(x) perform the OR and XOR operations
respectively.
For example, if x is a 4 element group, then the statement:

y = or_all(x);

is equivalent to:

y = x[0] | x[1] | x[2] | x[3] ;

********************************************************************/

bool and_all(x)
bool x[];
   {
   int ii;
   bool y;
   y = x[0];
   for(ii = 1,size(x) - 1)
      y := y & x[ii];
   return y;
   }

bool or_all(x)
bool x[];
   {
   int ii;
   bool y;
   y = x[0];
   for(ii = 1,size(x) - 1)
      y := y | x[ii];
   return y;
   }

bool xor_all(x)
bool x[];
   {
   int ii;
   bool y;
   y = x[0];
   for(ii = 1,size(x) - 1)
      y := y ^ x[ii];
   return y;
   }




