
/********************************************************************
                      printeqn(outfile,var)

The  printeqn function outputs the results in equations format.
Basically, it does what the generic print() statement does, but since
it is coded in BOOL syntax, it can be modified and tailored to
specific needs.  Also, it can output the results to any output file,
while print() can only output the results to file number 1.
You have to call printeqn() separately for each variable.
If a group has to be printed, you can use the function gprinteqn()
which prints a group (by calling printeqn for each element)

The maximal number of characters in an output line can be configured
by changing the definition of charmax below.
The characters used to define the INVERT, OR, AND and end-of-statement
can be configured by changing the definitions of the preprocessor symbols:
.invchar, .orchar, .andchar and .endchar .
The default configuration defined below fits the definitions of ABEL
and CUPL:

#define .invchar "!"
#define .orchar " #"
#define .andchar " & "
#define .endchar "  ;\n"


If you want the output to be compatible with PALASM for example, you
need to replace the default definition with the following
definitions:

#define .invchar "/"
#define .orchar " +"
#define .andchar " * "
#define .endchar "  \n"

********************************************************************/

/* maximal number of characters in a line */
#define charmax  80 

/* default definitions (for ABEL or CUPL format) */
#define .invchar "!"
#define .orchar " #"
#define .andchar " & "
#define .endchar "  ;\n"


int printeqn(outfile,var)
bool var;        /* boolean variable to be printed */
int outfile;   /* output file number */
{
   bool pt,       /* contains the Product Term being printed */
        x;        /* on-set of the variable var */
   int 
       litnum,    /* no. of input literals in PT being printed */
       npt,       /* index of PT being printed */
       nlit,      /* index in pt of literal being printed */
       slit,      /* serial no. of literal being printed */
       charnum;   /* no. of characters already printed out on current line */

 /* initialization */
   x = one(var);

   if(get_status(var) & 0x400000) 
      printf(outfile,.invchar);  /* signal has inverted polarity (active low)*/ 
   printvarname(outfile,var);
   printf(outfile," = ");
   if (iszero(x)) {printf(outfile,"0  ;\n") ; return 0; }
   if (isone(x)) {printf(outfile,"1  ;\n") ; return 0; }
   while(getpt(pt,x,npt) == 0 )
   {
      if (npt > 0) 
      {printf(outfile,.orchar);  printf(outfile,"\n     ");}
      litnum = lit(pt); /* number of literals in pt */
      for(nlit = 0, litnum - 1)
      {
         slit = lser(pt,nlit);
         fileinfo(outfile,charnum,0);
         if(charnum + length(univ.igroup[slit]) >= charmax)
            printf(outfile,"\n");
         if(lstat(pt,slit) == 0) 
            printf(outfile,.invchar);
         printvarname(outfile,univ.igroup[slit]); /* print name of literal */
         if (nlit < litnum - 1) 
            printf(outfile,.andchar);
      }
      npt = npt + 1;
   }
   printf(outfile,.endchar);
   return 0;
}


/********************************************************************
                      gprinteqn(outfile,gvar)

The  gprinteqn function outputs the results in equations format.
gvar is the name of a boolean group whose elements will be printed.
The elements are printed in equations format one after the other
starting from element number 0, by calling printeqn() for each
element.   For more information refer to the header comment on the
function printeqn() at the top of this file.
********************************************************************/

int gprinteqn(outfile,gvar)
bool gvar[];        /* boolean group to be printed */
int outfile;        /* output file number */
{
   int ii;

   for(ii = 0, size(gvar) - 1)
       printeqn(outfile,gvar[ii]);
   return 0;
}


/********************************************************************
                      inthex(outfile,ad,q)

inthex outputs the results in standard INTEL HEX format suitable for
PROM programmers.
outfile is the output file number where the results are written.
ad is an input group which defines the address lines of the PROM,
where a[0] defines the least significant bit of the address.
q is a boolean group which defines the output lines of the PROM.
(Whatever boolean expressions assigned earlier to q, will be
generated by the PROM)

Example:

#include inthex.inc
bool q[8];
inp a4,a3,a2,a1,a0;
group add = [a4,a3,a2,a1,a0];
int outfile;
q = add | @0F0h;
outfile = 1;
inthex(outfile,add,q); 
end;

The above example shows a complete BOOL program that generates a PROM
with 5 address lines: a4...a0 and with 8 output lines: q[7]...q[0].
The PROM will have a '1' in all 4 high order bits q[7]...q[4], while
the 4 low order bits: q[3]...q[0] will show an exact copy of the
address bits a3...a0.
The results are written to output file number 1
(which is the  second file name on the invocation line.)
********************************************************************/

int inthex(outfile,ad,q)
/* outputs results in INTEL-HEX format */
int outfile;   /* output file number */
inp  ad[];     /* address bits */
bool q[];      /* data bits */
{
   int ii,
       outbyte, /* data byte */
       bytesum, /* sum of all bytes in the line */
       len,     /* number of data bytes in the line */
       addcnt,  /* address counter */
       asize,   /* no. of elements (bits) in ad */
       addsize; /* no. of bytes in the PROM */
   
   asize = size(ad);
   addsize = 1;
   for ( ii = 0, asize-1 ) {addsize = addsize * 2 ;}
   addcnt = 0;
   while ( addcnt < addsize)
   {
      len = addsize - addcnt;
      if(len > 20h) len = 20h;
      printf(outfile,":%2h%4h00",len,addcnt); /* record mark, 
           no. of bytes in record, address, record type */
      bytesum = len + addcnt + (addcnt>>8);
      for (ii = 0,len-1)
      {
         values (ad = @addcnt);
         outbyte = eval(q) ;
         bytesum = bytesum + outbyte ;
         printf(outfile,"%2h",outbyte);
         addcnt = addcnt + 1;
      }
      printf(outfile,"%2h\n",-bytesum & 0ffh);   /* checksum , end of line */
   }
   printf(outfile,":00000001FF\n"); /* end of data record */
   return 0;
} 



   

