
/* design of a 4x4 serial/parallel multiplier 
pa,pb are parallel ports from which two 4-bit input words are loaded
into the input registers A and B. 
The circuit is triggered by the input signal com.
The signal busy, when 0, indicates that the register ACC holds
a valid result.
The A register is loaded only once and holds its value during 
the multiplication.
The B and the ACC registers are shifted up during the multiply steps.
Only bits that contain relevant data in B are shifted and the others remain
don't care.

The results are minimized and printed in boolean equations form.
As a last step the circuit is simulated by testing a few cases.
*/


/******************* declarations ********************/
ff ACC[8];  /* accumulator */

ff A[4]; /* holds value of pa */

ff B[4]; /* holds value of pb */


inp pa[4], /* parallel port a */
    pb[4], /* parallel port b */

    com;  /* when 1 it loads the registers and starts multiplication */

ff busy; /* is 1 when circuit is busy during multiplication,
            becomes 0 when circuit is ready */

ff st[3];  /* state register */

int wait, step1,step2,step3,step4;

int ii,jj,IA[20], IB[20];  /* used for simulation */


/*********** definition of circuit behavior ************/

/* state assignments */
wait = 0; step1 = 1; step2 = 2; step3 = 3; step4 = 4;

busy:d = 1;

.switch(st)
{
    @wait:
        if(com == 0)
        {
           ACC:D = ACC;  /* ACC retains its value until loaded with new value */
           busy:d := 0; 
           stay;
        }
        else  
        /* load registers from parallel ports and clear ACC*/
        {
           A:d = pa;
           B:d = pb;
           ACC:d = @0;
           jump step1;
        }
        
    @step1:
        A:d = A;  /* A retains its value */
        if(ACC == @0)  /* the case ACC != @0 cannot happen,
                          because ACC was loaded with 0 in the previous state
                          therefore it's a Don't Care */
        {
            if(B[3]) 
                ACC:d = A;  
            else 
                ACC:d = @0; 
                
            B[3] : d = B[2];  /* shift B register */
            B[2]:d = B[1];  
            B[1]:d = B[0];
            jump step2;
        }
        
    @step2:
        A:d = A;  /* A retains its value */
        if(B[3]) 
            ACC :d = gr(8,ACC,0) + A;  /* shift left and add */
        else 
            ACC:d = gr(8,ACC,0);      /* shift only */

        B[3]:d = B[2];  /* shift B register */
        B [ 2 ] : d  = B[1];  
        jump step3;

    @step3:
        A:d = A;  /* A retains its value */
        if(B[3]) 
            ACC:d = gr(8,ACC,0) + A;  /* shift left and add */
        else 
            ACC:d = gr(8,ACC,0);      /* shift only */
        B[3]:d = B [2];  /* shift B register */
        jump step4;

    @step4:
        if(B[3]) 
            ACC:d = gr(8,ACC,0) + A;  /* shift left and add */
        else 
            ACC:d = gr(8,ACC,0);      /* shift only */
        
        busy : d := 0;  /* result will be ready on next clock */
        jump wait;
}

/****************minimizing and printing results ***************/
minimize(ACC,busy,A,B,st);
print(ACC,busy,A,B,st);

/******************* simulating **********************/

IA[0] = 5; IB[0] = 6;
IA[1] = 0; IB[1] = 3;
IA[2] = 15; IB[2] = 15;
IA[3] = 12; IB[3] = 5;

#define prnt  printf(2,"com=%b pa=%2u pb=%2u A=%2u B=%2u st=%2u  busy=%b ACC=%3u\n", com,pa,pb,A,B,st,busy,ACC)

for(ii = 0,3)
{
   printf(2,"\nloading new value\n");
   values(pa = @IA[ii], pb = @IB[ii] , com = 1);
   prnt;
   clockff();
   values( com = 0);
   prnt;
   for(jj = 0,4)
   {
       clockff();
       prnt;
   }
}


end;






